<?php


namespace ContainerCheck;

use Concrete\Core\Asset\Asset;
use Concrete\Core\Entity\Page\Container;
use Concrete\Core\Package\PackageService;
use Concrete\Core\Page\Container\TemplateLocator;
use Concrete\Core\Page\Page;
use Concrete\Core\Support\Facade\Application;
use Concrete\Core\Asset\AssetList;
use Doctrine\ORM\EntityManagerInterface;

defined('C5_EXECUTE') or die('Access Denied.');

/*
 * Software by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class ContainerChecker
{
    protected $container;
    protected $raw_container_html;

    public function __construct($container)
    {
        $this->container = $container;
        $this->app = Application::getFacadeApplication();
    }

    /*
     * HTML without area names
     */
    public function getContainerHTML()
    {
        return $this->cleanup_container_html($this->getContainerRAWHTML());
    }


    /*
     * Strip it down to only what is needed to represent the container
     */
    protected function cleanup_container_html($container_html)
    {
        /*
         * Nothing before the first <div>
         */
        $container_html = preg_replace("/^.*?<div/si", '<div', $container_html);

        /*
         * No php tags, though htmlawed should also catch such
         */
        $container_html = preg_replace("/<\?.*?\?>/si", '', $container_html);

        /*
         * Then a further cleanup with htmlawed
         */
        $limitations = [
            'safe'           => 1, // no scripts or handlers
            'tidy'           => '4s', // format, 4 spaces indentation
            'elements'       => 'div', // only <div> elements
            'deny_attribute' => '* -class' // only class attributes
        ];

        $container_html = htmLawed($container_html, $limitations);

        return $container_html;
    }

    /*
    * HTML with area names
    */
    public function getContainerHTMLWithAreas()
    {
        return $this->cleanup_container_html_keeping_areas($this->getContainerRAWHTML());
    }

    /*
     * This is the tricky variant because we want to keep area names, but it has to cope with
     * code tricks such as collapsable areas
     */
    protected function cleanup_container_html_keeping_areas($container_html)
    {
        /*
         * Nothing before the first <div>
         */
        $container_html = preg_replace("/^.*?<div/si", '<div', $container_html);

        /*
         * Find $variable used in any area name declarations, note the identifier, then replace ->display
         * of each such $variable with the area name. This needs to be non-greedy and only act within single
         * <? ... ?>
         */
        if (preg_match_all("/<\?.*?\\\$([\w]+)\s*=.*?ContainerArea\(.*?\'([\w\s]+)\'/si", $container_html, $m)) {
            foreach ($m[1] as $ix => $area_var) {
                $area_name = $m[2][$ix];
                $container_html = preg_replace("/<\?[^\?]*?" . $area_var . "\s*\-\>\s*display.*?\?>/si", $area_name, $container_html, 1);
            }
        }
        /*
         * Strip any remaining php tags, though htmlawed should also catch such
         */
        $container_html = preg_replace("/<\?.*?\?>/si", '', $container_html);

        /*
         * Then a further cleanup with htmlawed
         */
        $limitations = [
            'safe'           => 1, // no scripts or handlers
            'tidy'           => '4s', // format, 4 spaces indentation
            'elements'       => 'div', // only <div> elements
            'deny_attribute' => '* -class' // only class attributes
        ];

        $container_html = htmLawed($container_html, $limitations);

        return $container_html;
    }


    public function getContainerHTMLWithAreasNoClasses()
    {
        return $this->cleanup_container_html_no_classes($this->getContainerRAWHTML());
    }
    protected function cleanup_container_html_no_classes($container_html)
    {
        $container_html = $this->cleanup_container_html_keeping_areas($container_html);
        $limitations = [
            'safe'           => 1, // no scripts or handlers
            'tidy'           => '4s', // format, 4 spaces indentation
            'elements'       => 'div', // only <div> elements
            'deny_attribute' => '*' // no attributes
        ];

        $container_html = htmLawed($container_html, $limitations);
        return $container_html;
    }

    /*
     * Locate a container's file, read it, clean up the html and return it.
     * Only read teh file once, cache in a variable
     */
    public function getContainerRawHTML()
    {
        if (!empty($this->raw_container_html)) {
            return $this->raw_container_html;
        }
        if (empty($this->raw_container_html)) {
            $locator = $this->app->make(TemplateLocator::class);
            $home = Page::getByID(Page::getHomePageID());
            $fh = $this->app->make('helper/file');

            $container_file = $locator->getFileToRender($home, $this->container);
            $this->raw_container_html = trim($fh->getContents($container_file));
        }
        return $this->raw_container_html;
    }



    /*
     * Assets specific to container checker
     * Usage:
     *  ContainerChecker::register_assets();
     *  $this->requireAsset('container_check');
     */
    static public function register_assets()
    {
        $pkg = Application::getFacadeApplication()->make(PackageService::class)->getByHandle('jl_container_check');
        $al = AssetList::getInstance();

        $al->register('css',
                      'container_check',
                      'css/container_check.css',
                      [
                          'version'  => $pkg->getPackageVersion(),
                          'position' => Asset::ASSET_POSITION_HEADER,
                          'minify'   => true,
                          'combine'  => true
                      ],
                      $pkg
        );

        $al->register('javascript',
                      'container_check',
                      'js/container_check.js',
                      [
                          'version'  => $pkg->getPackageVersion(),
                          'position' => Asset::ASSET_POSITION_FOOTER,
                          'minify'   => true,
                          'combine'  => true
                      ],
                      $pkg
        );

        $al->registerGroup('container_check', [
            ['css', 'container_check'],
            ['javascript', 'jquery'],
            ['javascript', 'container_check'],
        ]);
    }

    /*
     * Purloined from the dashboard container list page
     */
    static public function list_containers()
    {
        $entityManager = Application::getFacadeApplication()->make(EntityManagerInterface::class);
        $containers = $entityManager->getRepository(Container::class)->findBy([], ['containerName' => 'asc']);

        usort($containers, function ($a, $b) {
            $ah = $a->getContainerName();
            $bh = $b->getContainerName();
            if ($ah < $bh) {
                return -1;
            }
            if ($ah > $bh) {
                return 1;
            }
        });
        return $containers;
    }


}